
clinical_data_dictionary = {
   
    'tobacco_smoking_history':
        {
            '1': 'Lifelong Non-smoker',
            '2': 'Current smoker',
            '3': 'Current reformed smoker for > 15 years',
            '4': 'Current reformed smoker for < or = 15 years',
            '5': 'Current Reformed Smoker, Duration Not Specified',

        }
}