TCGA_NIH_REQUEST_ADDRESS = "http://tcga-data.nci.nih.gov/tcga/damws/jobprocess/json"

TCGA_TICKET_ID_FIELD = 'ticket'
TCGA_SUBMISSION_TIME_FIELD = 'submission-time'
TCGA_ESTIMATED_SIZE_FIELD = 'estimated-size'
TCGA_STATUS_CHECK_URL_FIELD = 'status-check-url'

REQUEST_ADDRESS = TCGA_NIH_REQUEST_ADDRESS
JOB_STATUS_ADDRESS = "http://tcga-data.nci.nih.gov/tcga/damws/jobprocess/json/ticket/"
